/* $Id: XtConf.cpp,v 1.1 2006/02/02 15:37:57 jsato Exp $ */
/*
 * Copyright (C) 2006 WIDE Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * 2006/02/03 jsato: created
 *
 */


#include <qprocess.h>
#include <qregexp.h>

#include "qxcgroup.h"
#include "XtConf.h"


XtConf::XtConf()
{
  groupName     = QString("");
  groupServer   = QString("");
  groupPort     = DEF_XCGRP_PORT;
  vicCmd        = DEF_VIC_CMD;
  vicPort       = DEF_VIC_PORT;
  vicTTL        = DEF_TTL;
  ratCmd        = DEF_RAT_CMD;
  ratPort       = DEF_RAT_PORT;
  ratTTL        = DEF_TTL;
  keepAliveTime = DEF_KALIVE_TIME;
  updateTime    = DEF_UPDATE_TIME;
  ifName        = QString("");
  ifAddr        = QString("");
  ifPrefix      = 0;

  /* JSATO
   * - just testing
   */
  os_type       = DEF_OS_TYPE;

  return;
}


QString XtConf::getGroupName()
{
  return groupName;
}


const char *XtConf::getGroupNameChar()
{
  return (const char *)groupName.data();
}


QString XtConf::getGroupServer()
{
  return groupServer;
}


const char *XtConf::getGroupServerChar()
{
  return (const char *)groupServer.data();
}


Q_UINT16 XtConf::getGroupPort()
{
  return groupPort;
}


QString XtConf::getVicCmd()
{
  return vicCmd;
}


const char *XtConf::getVicCmdChar()
{
  return (const char *)vicCmd.data();
}


Q_UINT16 XtConf::getVicPort()
{
  return vicPort;
}


Q_UINT16 XtConf::getVicTTL()
{
  return vicTTL;
}


QString XtConf::getRatCmd()
{
  return ratCmd;
}


const char *XtConf::getRatCmdChar()
{
  return (const char *)ratCmd.data();
}


Q_UINT16 XtConf::getRatPort()
{
  return ratPort;
}


Q_UINT16 XtConf::getRatTTL()
{
  return ratTTL;
}


Q_UINT32 XtConf::getKeepAliveTime()
{
  return keepAliveTime;
}


Q_UINT32 XtConf::getUpdateTime()
{
  return updateTime;
}


QString XtConf::getIfName()
{
  return ifName;
}


const char *XtConf::getIfNameChar()
{
  return (const char *)ifName.data();
}


QString XtConf::getIfAddr()
{
  return ifAddr;
}


QString XtConf::getIfAddr(QString ifname)
{
  if(ifName == ifname){
    if(ifAddr.isEmpty() == TRUE)
      ifAddr = getlocal(ifname);
  }else{
    ifAddr = getlocal(ifname);
    ifName = ifname;
  }

  return ifAddr;
}


Q_UINT16 XtConf::getIfPrefix()
{
  return ifPrefix;
}


QValueList<QString> XtConf::getIfNameAll()
{
  return getifname();
}


bool XtConf::hasIfAddr()
{
  if(ifAddr.isEmpty() == TRUE)
    return FALSE;
  else
    return TRUE;
}


bool XtConf::hasIfPrefix()
{
  if(ifPrefix == 0)
    return FALSE;
  else
    return TRUE;
}


void XtConf::setGroupName(QString val)
{
  groupName = val;
  return;
}


void XtConf::setGroupServer(QString val)
{
  groupServer = val;
  return;
}


void XtConf::setGroupPort(Q_UINT16 val)
{
  groupPort = val;
  return;
}


void XtConf::setVicCmd(QString val)
{
  vicCmd = val;
  return;
}


void XtConf::setVicPort(Q_UINT16 val)
{
  vicPort = val;
  return;
}


void XtConf::setVicTTL(Q_UINT16 val)
{
  vicTTL = val;
  return;
}


void XtConf::setRatCmd(QString val)
{
  ratCmd = val;
  return;
}


void XtConf::setRatPort(Q_UINT16 val)
{
  ratPort = val;
  return;
}


void XtConf::setRatTTL(Q_UINT16 val)
{
  ratTTL = val;
  return;
}


void XtConf::setKeepAliveTime(Q_UINT32 val)
{
  keepAliveTime = val;
  return;
}


void XtConf::setUpdateTime(Q_UINT32 val)
{
  updateTime = val;
  return;
}


void XtConf::setIfName(QString val)
{
  if(ifName == val){
    if(ifAddr.isEmpty() == TRUE)
      ifAddr = getlocal(val);
  }else{
    ifAddr = getlocal(val);
    ifName = val;
  }
  return;
}


QValueList<QString> XtConf::getifname()
{
  QValueList<QString> ifname;
  QProcess *proc = new QProcess();

  if(os_type == "win32"){
    MSGVRB("Sorry, Now implementing...");
    delete proc;
    return ifname;
#if 0
    proc->setArguments("ipv6");
    proc->addArgument("if");
#endif
  }else if(os_type == "linux"){
    proc->setArguments("cat");
    proc->addArgument("/proc/net/if_inet6");
  }else if(os_type == "bsd"){
    proc->setArguments("/sbin/ifconfig");
    proc->addArgument("-ul");
  }else{
    MSGERR("No OS Type Specified");
    delete proc;
    return ifname;
  }

  if(!proc->start()){
    if(os_type == "win32")
      MSGERR("unable to run \"ipv6 if\"");
    else if(os_type == "linux")
      MSGERR("unable to run \"cat /proc/net/if_inet6\"");
    else if(os_type == "bsd")
      MSGERR("unable to run \"/sbin/ifconfig\"");

    delete proc;
    return ifname;
  }

  /* JSATO
   * - just have it stable
   */
  usleep(500);

  /* Error has ocurred */
  if(proc->canReadLineStderr() == TRUE){
    QString line("");
    while((line = proc->readLineStderr()) != QString::null)
      MSGVRB("%s", line.data());

    delete proc;
    return ifname;
  }

  /* Parse output */
  if(proc->canReadLineStdout() == TRUE){
    QString line("");
    while((line = proc->readLineStdout()) != QString::null){
      line = line.simplifyWhiteSpace();

      if(os_type == "win32"){         //Windows "ipv6 if" command
        MSGVRB("Sorry, Now implementing...");
        delete proc;
        return ifname;
      }else if(os_type == "linux"){   //Linux "cat /proc/net/if_inet6" command
        QString if_v6 = line.section(" ", 5, 5);
        if(if_v6 == "lo")
          continue;

        bool found = FALSE;
        QValueList<QString>::iterator it;
        for(it = ifname.begin(); it != ifname.end(); it++){
          if((*it).compare(if_v6) == 0)
            found = TRUE;
        }

        if(found == FALSE)
          ifname.push_back(if_v6);

      }else if(os_type == "bsd"){     //*BSD "/sbin/ifconfig" command
        int i = 0;
        QString if_v6 = line.section(" ", 0, 0);
        for(; if_v6.isNull() == FALSE; i++, if_v6 = line.section(" ", i, i)){
          if(if_v6 == "lo0")
            continue;

          ifname.push_back(if_v6);
        }
        break;
      }
    }
  }

  delete proc;
  return ifname;
}


QString XtConf::getlocal(QString ifname)
{
  if(ifname.isEmpty() == TRUE){
    MSGERR("No interface name specified");
    return ifname;
  }

  QString addr("");
  QString prefix("");

  /* Create QProcess instance */
  QProcess *proc = new QProcess();

  /* Set arguments of each OS */
  if(os_type == "win32"){
    MSGVRB("Sorry, Now implementing...");
    delete proc;
    return addr;
#if 0
    proc->setArguments("ipv6");
    proc->addArgument("if");
    QString ifindex("");
    proc->addArgument(ifindex.sprintf("%d", ifname));
#endif
  }else if(os_type == "linux"){
    proc->setArguments("/sbin/ifconfig");
    proc->addArgument(ifname);
  }else if(os_type == "bsd"){
    proc->setArguments("/sbin/ifconfig");
    proc->addArgument(ifname);
    proc->addArgument("inet6");
  }else{
    MSGERR("No OS Type Specified");
    delete proc;
    return addr;
  }

  /* Run showing interface as external process */
  if(!proc->start()){
    if(os_type == "win32")
      MSGERR("unable to run \"ipv6 if\"");
    else
      MSGERR("unable to run \"/sbin/ifconfig\"");

    delete proc;
    return addr;
  }

  /* JSATO
   * - just testing
   */
  usleep(500);

  /* Error has ocurred */
  if(proc->canReadLineStderr() == TRUE){
    QString line("");
    while((line = proc->readLineStderr()) != QString::null)
      MSGVRB("%s", line.data());

    delete proc;
    return addr;
  }

  /* Parse output */
  if(proc->canReadLineStdout() == TRUE){
    QString line("");
    while((line = proc->readLineStdout()) != QString::null){
      line = line.simplifyWhiteSpace();

      if(os_type == "win32"){         //Windows "ipv6 if" command
        MSGVRB("Sorry, Now implementing...");
        delete proc;
        return addr;
#if 0
        if(line.section(" ", 0, 0) != "preferred")
          continue;

        if(line.section(" ", 1, 1) != "global")
          continue;
        else{
          QString addr_v6 = line.section(" ", 2, 2);
          addr_v6.remove(",");
          addr = addr_v6;
          break;
        }
#endif
      }else if(os_type == "linux"){   //Linux "/sbin/ifconfig" command
        if(line.section(" ", 0, 0) != "inet6")
          continue;

        if(line.section(" ", 3, 3) != "Scope:Global")
          continue;
        else{
          QString addr_v6 = line.section(" ", 2, 2);
          addr_v6.replace("/", " ");
          addr   = addr_v6.section(" ", 0, 0);
          prefix = addr_v6.section(" ", 1, 1);
          break;
        }
      }else if(os_type == "bsd"){     //*BSD "/sbin/ifconfig" command
        if(line.section(" ", 0, 0) != "inet6")
          continue;

        QString addr_v6 = line.section(" ", 1, 1);
        if(addr_v6.find(QRegExp("^fe80"), 0) < 0)
          continue;
        else if(addr_v6.find(QRegExp("^::1"), 0) < 0)
          continue;
        else if(addr_v6.find(QRegExp("^fe[c-f]"), 0) < 0)
          continue;
        else{
          addr   = addr_v6;
          prefix = line.section(" ", 3, 3);
          break;
        }
      }
    }
  }

  /* Set to class-scope variable */
  ifPrefix = prefix.toUShort(); 

  delete proc;
  return addr;
}
