/* $Id: XtHttp.cpp,v 1.1 2006/02/02 15:37:57 jsato Exp $ */
/*
 * Copyright (C) 2006 WIDE Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * 2006/02/03 jsato: created
 *
 */


#include <qdns.h>

#include "qxcgroup.h"
#include "XtHttp.h"



XtHttp::XtHttp(QString srv, Q_UINT16 port)
{
  sock_srv  = srv;
  sock_port = port;
  sock     = 0;
  sock_dns = 0;
  memset(recv_buf, 0, 2048);
  recv_len = 0;
}


XtHttp::~XtHttp()
{
  if(sock_dns != 0)
    delete sock_dns;

  if(sock != 0){
    sock->close();
    delete sock;
  }
}


void XtHttp::get(QString url, bool grp_get)
{
  http_url = url;

  sock     = new QSocketDevice(QSocketDevice::Stream,
                               QSocketDevice::IPv6, 0);
  sock_dns = new QDns(sock_srv, QDns::Aaaa);

  if(grp_get == TRUE)
    connect(sock_dns, SIGNAL(resultsReady()), this, SLOT(httpGetMember()));
  else
    connect(sock_dns, SIGNAL(resultsReady()), this, SLOT(httpGet()));

  return;
}


void XtHttp::stop()
{
  if(sock_dns != 0)
    delete sock_dns;

  if(sock != 0){
    sock->close();
    delete sock;
  }

  sock     = 0;
  sock_dns = 0;
  return;
}


void XtHttp::httpGet()
{
  Q_UINT8 ret = doHttp();
  if(ret == XCGRP_OK)
    emit sigSendEnd();
  else
    emit sigSendErr(ret);

  return;
}


void XtHttp::httpGetMember()
{
  Q_UINT8 ret = doHttp();
  if(ret == XCGRP_OK)
    emit sigSendData((const char *)recv_buf, recv_len);
  else
    emit sigSendErr(ret);

  return;
}


void XtHttp::httpUpdate(QString srv, Q_UINT16 port)
{
  sock_srv  = srv;
  sock_port = port;
  return;
}


Q_UINT8 XtHttp::doHttp()
{
  int sock_fd = -1;
  QHostAddress sock_v6addr;

  /* Get server sockaddr from AAAA record */
  QValueList<QHostAddress> list = sock_dns->addresses();
  QValueList<QHostAddress>::Iterator it = list.begin();
  while(it != list.end()){
    if((*it).isIPv6Address() == TRUE){
      sock_v6addr = (*it).toIPv6Address();
      break;
    }
    ++it;
  }

  /* convert from string to sockaddr */
  if(sock_v6addr.isNull() == TRUE){
    if(sock_v6addr.setAddress(sock_srv) == FALSE){
      stopHttp();
      return XCGRP_E_BAD_ADDR;
    }
  }

  sock_fd = sock->socket();
  if(sock_fd < 0){
    stopHttp();
    return XCGRP_E_SOCKET;
  }

  sock->setAddressReusable(TRUE);

  MSGDBG("connect to %s/%d", sock_srv.data(), sock_port);

  if(sock->connect(sock_v6addr, sock_port) == FALSE){
    sock->close();
    stopHttp();
    return XCGRP_E_CONNECT;
  }

  QString *hdr = new QString("");
  httpBuild(hdr, http_url);

  Q_LONG len = sock->writeBlock((const char *)hdr->data(), hdr->length());
  if(len < 0){
    sock->close();
    stopHttp();
    return XCGRP_E_SEND;
  }

  memset(recv_buf, 0, 2048);
  recv_len = 0;
  len = 0;

  len = sock->readBlock(recv_buf, (Q_ULONG)2048);
  if(len < 0){
    sock->close();
    stopHttp();
    return XCGRP_E_RECV;
  }
  recv_len = len;

  Q_UINT8 rep = httpResp((const char *)recv_buf, recv_len);
  if(rep != XCGRP_OK){
    sock->close();
    stopHttp();
    return rep;
  }

  stopHttp();
  return XCGRP_OK;
}


void XtHttp::stopHttp()
{
  delete sock;
  delete sock_dns;
  sock     = 0;
  sock_dns = 0;
  return;
}


void XtHttp::httpBuild(QString *httpHdr, QString url)
{
  QString hdr("");
  hdr = QString::fromLatin1("GET ") + url +
        QString::fromLatin1("\nHTTP/1.1") +
        QString::fromLatin1("\nUser-Agent:XCGroup/Qt 0.0") +
//        QString::fromLatin1("\nContent-Type: text/plain;") +
        QString::fromLatin1("\n\n" );
  hdr.replace(QString::fromLatin1("\n"),
              QString::fromLatin1("\r\n"));

  *httpHdr = hdr;
  return;
}


Q_UINT8 XtHttp::httpResp(const char *data, Q_ULONG len)
{
  QString rep(data);

  if(rep.contains("ERROR"))
    return XCGRP_E_HTTP;

  if(rep.contains("400 Bad Request"))
    return XCGRP_E_HTTP_400;

  if(rep.contains("401 Unauthorized"))
    return XCGRP_E_HTTP_401;

  if(rep.contains("403 Forbidden"))
    return XCGRP_E_HTTP_403;

  if(rep.contains("404 Not Found"))
    return XCGRP_E_HTTP_404;

  if(rep.contains("405 Method Not Allowed"))
    return XCGRP_E_HTTP_405;

  if(rep.contains("406 Not Acceptable"))
    return XCGRP_E_HTTP_406;

  if(rep.contains("407 Proxy Authentication Required"))
    return XCGRP_E_HTTP_407;

  if(rep.contains("408 Request Timeout"))
    return XCGRP_E_HTTP_408;

  if(rep.contains("500 Internal Sever Error"))
    return XCGRP_E_HTTP_500;

  if(rep.contains("501 Not Implemented"))
    return XCGRP_E_HTTP_501;

  if(rep.contains("502 Bad Gateway"))
    return XCGRP_E_HTTP_502;

  if(rep.contains("503 Service Unavailable"))
    return XCGRP_E_HTTP_503;

  if(rep.contains("504 Gateway Timeout"))
    return XCGRP_E_HTTP_504;

  if(rep.contains("505 HTTP Version Not Supported"))
    return XCGRP_E_HTTP_505;

  if(rep.contains("Server Too Busy"))
    return XCGRP_E_HTTP_BUSY;

  return XCGRP_OK;
}
