/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
/* $Id: XtMain.ui.h,v 1.1 2006/02/02 15:37:57 jsato Exp $ */
/*
 * Copyright (C) 2006 WIDE Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * 2006/02/03 jsato: created
 *
 */


#include <stdlib.h>


void XtMain::init()
{
  joined   = FALSE;
  run_vic  = FALSE;
  run_rat  = FALSE;
  num_item = 0;
  conf     = new XtConf();
  rcFileName = new QString(getenv("HOME"));
  rcFileName->append("/.qxcgrouprc");

  /* Read qxcgroup configuration from $HOME/.qxcgrouprc */
  QFile rcFile(*rcFileName);
  if(rcFile.exists() == TRUE)
    readConf(*rcFileName);

  /* Create xcgroup instance */
  xcgrp = new XtXcgrp(conf);

  /* Register SLOT (XtXcgrp -> XtMain) */
  connect(xcgrp, SIGNAL(sigAddMember(XtMember)),
          this, SLOT(addMember(XtMember)));
  connect(xcgrp, SIGNAL(sigAddToMember(XtMember, XtMember)),
          this, SLOT(addToMember(XtMember, XtMember)));
  connect(xcgrp, SIGNAL(sigDelMember(QString)),
          this, SLOT(delMember(QString)));
  connect(xcgrp, SIGNAL(sigUpdateMember(QValueList<XtMember>)),
          this, SLOT(updateMember(QValueList<XtMember>)));

  /* Register SLOT (XtMain -> XtXcgrp) */
  connect(this, SIGNAL(sigConfMember(XtMember)),
          xcgrp, SLOT(confMember(XtMember)));
  connect(this, SIGNAL(sigConfUpdate()),
          xcgrp, SLOT(confUpdate()));

  /* Create 2 timers to send join command and to update member list */
  keepAliveTimer = new QTimer(this);
  updateTimer = new QTimer(this);
  connect(keepAliveTimer, SIGNAL(timeout()), this, SLOT(doKeepAlive()));
  connect(updateTimer, SIGNAL(timeout()), this, SLOT(doUpdate()));

  /* JSATO
   *  - the followings are under development
   */
  fileOpenAction->setEnabled(FALSE);
  fileSaveAction->setEnabled(FALSE);
  fileSaveAsAction->setEnabled(FALSE);

  XtMainJoinBtn->setEnabled(TRUE);
  XtMainLeaveBtn->setEnabled(FALSE);
  toolsJoinGroupAction->setEnabled(TRUE);
  toolsLeaveGroupAction->setEnabled(FALSE);

  XtMainGroupNameValue->setText(conf->getGroupName());
  XtMainGroupServerValue->setText(conf->getGroupServer());
  XtMainGroupPortValue->setText(QString::number(conf->getGroupPort(), 10));

  JoinMember->setSorting(-1, FALSE);

  return;
}


void XtMain::destroy()
{
  /* Leave Group */
  if(joined == TRUE){
    keepAliveTimer->stop();
    updateTimer->stop();
    xcgrp->leave();
  }

  /* Stop process of both VIC and RAT */
  if(run_vic == TRUE){
    if(procVic->isRunning() == TRUE){
      MSGVRB("Finished VIC");
      procVic->tryTerminate();
      QTimer::singleShot(1000, procVic, SLOT(kill()));
    }
  }

  if(run_rat == TRUE){
    if(procRat->isRunning() == TRUE){
      MSGVRB("Finished RAT");
      procRat->tryTerminate();
      QTimer::singleShot(1000, procRat, SLOT(kill()));
    }
  }

  /* Store configuration to $HOME/.qxcgrouprc */
  saveConf(*rcFileName);

  delete keepAliveTimer;
  delete updateTimer;
  delete xcgrp;
  delete conf;

  this->close();
  return;
}


void XtMain::readConf(QString file)
{
  /* Read qxcgroup configuration from $HOME/.qxcgrouprc */
  QFile rcFile(file);
  if(rcFile.open(IO_ReadOnly) == TRUE){
    QTextStream stream(&rcFile);
    while(!stream.eof()){
      QString line = stream.readLine();
      line = line.simplifyWhiteSpace();
      QString param = line.section(" ", 0, 0);
      QString value = line.section(" ", 1, 1);

      if(param == QString("group_name"))
        conf->setGroupName(value);
      else if(param == QString("group_server"))
        conf->setGroupServer(value);
      else if(param == QString("group_port"))
        conf->setGroupPort(value.toUShort());
      else if(param == QString("vic_cmd"))
        conf->setVicCmd(value);
      else if(param == QString("vic_port"))
        conf->setVicPort(value.toUShort());
      else if(param == QString("vic_ttl"))
        conf->setVicTTL(value.toUShort());
      else if(param == QString("rat_cmd"))
        conf->setRatCmd(value);
      else if(param == QString("rat_port"))
        conf->setRatPort(value.toUShort());
      else if(param == QString("rat_ttl"))
        conf->setRatTTL(value.toUShort());
      else if(param == QString("keep_alive"))
        conf->setKeepAliveTime(value.toUInt());
      else if(param == QString("update_time"))
        conf->setUpdateTime(value.toUInt());
      else if(param == QString("if_name"))
        conf->setIfName(value);
    }
    rcFile.close();
  }
  return;
}


void XtMain::saveConf(QString file)
{
  QFile rcFile(file);
  if(rcFile.exists() == TRUE)
    rcFile.remove();

  if(rcFile.open(IO_WriteOnly) == TRUE){
    QTextStream stream(&rcFile);
    QString rc_data;
    rc_data.sprintf("group_name %s\n", conf->getGroupNameChar());
    stream << rc_data;
    rc_data.sprintf("group_server %s\n", conf->getGroupServerChar());
    stream << rc_data;
    rc_data.sprintf("group_port %d\n", conf->getGroupPort());
    stream << rc_data;
    rc_data.sprintf("vic_cmd %s\n", conf->getVicCmdChar());
    stream << rc_data;
    rc_data.sprintf("vic_port %d\n", conf->getVicPort());
    stream << rc_data;
    rc_data.sprintf("vic_ttl %d\n", conf->getVicTTL());
    stream << rc_data;
    rc_data.sprintf("rat_cmd %s\n", conf->getRatCmdChar());
    stream << rc_data;
    rc_data.sprintf("rat_port %d\n", conf->getRatPort());
    stream << rc_data;
    rc_data.sprintf("rat_ttl %d\n", conf->getRatTTL());
    stream << rc_data;
    rc_data.sprintf("keep_alive %d\n", conf->getKeepAliveTime());
    stream << rc_data;
    rc_data.sprintf("update_time %d\n", conf->getUpdateTime());
    stream << rc_data;
    rc_data.sprintf("if_name %s\n", conf->getIfNameChar());
    stream << rc_data;
    rcFile.close();
  }
  return;
}


void XtMain::fileOpen()
{
#if 0
  QString seek_dir;
  QFileInfo fi(confUserMngValue->displayText());
  if(fi.exists())
    seek_dir = fi.filePath();

  QString new_file = QFileDialog::getOpenFileName(
    seek_dir,
    "user management(spkuser);;" "text file(*.txt);;" "all files(*.*)",
    this,
    "open file dialog",
    "Choose a qxcgroup configuration file");

  if(!new_file.isEmpty()){
    this->confUserMngValue->setText(new_file);
    this->repaint();
  }
#endif
  return;
}


void XtMain::fileSave()
{
//  saveConf(*rcFileName);
  return;
}


void XtMain::fileSaveAs()
{

}


void XtMain::fileExit()
{
  destroy();
  return;
}


void XtMain::editCut()
{

}


void XtMain::editCopy()
{

}


void XtMain::editPaste()
{

}


void XtMain::editPreferences()
{
  XtPref *pref = new XtPref();

  connect(pref, SIGNAL(sigSetGroupName(QString)),
          this, SLOT(setGroupName(QString)));

  connect(pref, SIGNAL(sigSetGroupServer(QString)),
          this, SLOT(setGroupServer(QString)));

  connect(pref, SIGNAL(sigSetGroupPort(Q_UINT16)),
          this, SLOT(setGroupPort(Q_UINT16)));

  connect(pref, SIGNAL(sigSetVicCmd(QString)),
          this, SLOT(setVicCmd(QString)));

  connect(pref, SIGNAL(sigSetVicPort(Q_UINT16)),
          this, SLOT(setVicPort(Q_UINT16)));

  connect(pref, SIGNAL(sigSetVicTTL(Q_UINT16)),
          this, SLOT(setVicTTL(Q_UINT16)));

  connect(pref, SIGNAL(sigSetRatCmd(QString)),
          this, SLOT(setRatCmd(QString)));

  connect(pref, SIGNAL(sigSetRatPort(Q_UINT16)),
          this, SLOT(setRatPort(Q_UINT16)));

  connect(pref, SIGNAL(sigSetRatTTL(Q_UINT16)),
          this, SLOT(setRatTTL(Q_UINT16)));

  connect(pref, SIGNAL(sigSetKeepAliveTime(Q_UINT32)),
          this, SLOT(setKeepAliveTime(Q_UINT32)));

  connect(pref, SIGNAL(sigSetUpdateTime(Q_UINT32)),
          this, SLOT(setUpdateTime(Q_UINT32)));

  connect(pref, SIGNAL(sigSetIfName(QString)),
          this, SLOT(setIfName(QString)));

  pref->setConf(conf);
  pref->show();
  return;
}


void XtMain::helpIndex()
{

}


void XtMain::helpContents()
{

}


void XtMain::helpAbout()
{
  XtHelpAbout *help;
  help = new XtHelpAbout(this);
  help->show();
  return;
}


void XtMain::toolsRunVic()
{
  if(run_vic == FALSE){
    if(checkFile(conf->getVicCmd()) == FALSE){
      showMsgDlg("Cannot find vic command");
      return;
    }

    procVic = new QProcess(this);
    procVic->setArguments(conf->getVicCmd());
    procVic->addArgument("-n");
    procVic->addArgument("xcast6");
    /* JSATO
     * - the following does not work
     */
#if 0
    procVic->addArgument("-t");

    QString arg_ttl("");
    arg_ttl.number(conf->getVicTTL(), 10);
    procVic->addArgument(arg_ttl);
#endif

    QString arg_port("::1/");
    arg_port.append(arg_port.number(conf->getVicPort(), 10));
    procVic->addArgument(arg_port);

    connect(procVic, SIGNAL(readyReadStdout()),
            this, SLOT(vicReadStdout()));
    connect(procVic, SIGNAL(readyReadStderr()),
            this, SLOT(vicReadStderr()));
    connect(procVic, SIGNAL(processExited()),
            this, SLOT(vicExited()));

    if(!procVic->start()){
      showMsgDlg("Cannot run VIC");
      return;
    }
    run_vic = TRUE;
    toolsRunVicAction->setEnabled(FALSE);
  }else{
    if(procVic->isRunning() == FALSE){
      if(!procVic->start())
        showMsgDlg("Cannot run VIC");
    }
  }
  return;
}


void XtMain::toolsRunRat()
{
  if(run_rat == FALSE){
    if(checkFile(conf->getRatCmd()) == FALSE){
      showMsgDlg("Cannot find rat command");
      return;
    }

    procRat = new QProcess(this);
    procRat->setArguments(conf->getRatCmd());
    procRat->addArgument("-t");

    QString arg_ttl("");
    arg_ttl.number(conf->getRatTTL(), 10);
    procRat->addArgument(arg_ttl);

    QString arg_port("::1/");
    arg_port.append(arg_port.number(conf->getRatPort(), 10));
    procRat->addArgument(arg_port);

    connect(procRat, SIGNAL(readyReadStdout()),
            this, SLOT(ratReadStdout()));
    connect(procRat, SIGNAL(readyReadStderr()),
            this, SLOT(ratReadStderr()));
    connect(procRat, SIGNAL(processExited()),
            this, SLOT(ratExited()));

    if(!procRat->start()){
      showMsgDlg("Cannot run RAT");
      return;
    }
    run_rat = TRUE;
    toolsRunRatAction->setEnabled(FALSE);
  }else{
    if(procRat->isRunning() == FALSE){
      if(!procRat->start())
        showMsgDlg("Cannot run RAT");
    }
  }

  return;
}


void XtMain::vicReadStdout()
{
  QString msgStdout = procVic->readStdout();
  MSGVRB(msgStdout);
  return;
}


void XtMain::vicReadStderr()
{
  QString msgStderr = procVic->readStderr();
  MSGVRB(msgStderr);
  return;
}


void XtMain::vicExited()
{
  toolsRunVicAction->setEnabled(TRUE);
  delete procVic;
  procVic = 0;
  run_vic = FALSE;
  MSGVRB("vic process exited");
  return;
}


void XtMain::ratReadStdout()
{
  QString msgStdout = procRat->readStdout();
  MSGVRB(msgStdout);
  return;
}


void XtMain::ratReadStderr()
{
  QString msgStderr = procRat->readStderr();
  MSGVRB(msgStderr);
  return;
}


void XtMain::ratExited()
{
  toolsRunRatAction->setEnabled(TRUE);
  delete procRat;
  procRat = 0;
  run_rat = FALSE;
  MSGVRB("rat process exited");
  return;
}


void XtMain::XtMainJoinBtn_clicked()
{
  if(QString(conf->getGroupName()).isEmpty() == TRUE){
    showMsgDlg("No Group Name Specified");
    return;
  }

  if(QString(conf->getGroupServer()).isEmpty() == TRUE){
    showMsgDlg("No Group Server Specified");
    return;
  }

  if(conf->getGroupPort() == 0){
    showMsgDlg("No Group Port Specified");
    return;
  }

  if(conf->hasIfAddr() == FALSE){
    showMsgDlg("IPv6 Address to send is NOT Specified");
    return;
  }

  /* OK, it seems to get own address/prefix */
  emit sigConfUpdate();

  XtMainJoinBtn->setEnabled(FALSE);
  XtMainLeaveBtn->setEnabled(TRUE);
  toolsJoinGroupAction->setEnabled(FALSE);
  toolsLeaveGroupAction->setEnabled(TRUE);

  keepAliveTimer->start((conf->getKeepAliveTime()*1000), FALSE);
  updateTimer->start((conf->getUpdateTime()*1000), FALSE);
  joined = TRUE;
  xcgrp->join();

  return;
}


void XtMain::XtMainLeaveBtn_clicked()
{
  XtMainJoinBtn->setEnabled(TRUE);
  XtMainLeaveBtn->setEnabled(FALSE);
  toolsJoinGroupAction->setEnabled(TRUE);
  toolsLeaveGroupAction->setEnabled(FALSE);

  keepAliveTimer->stop();
  updateTimer->stop();
  joined = FALSE;
  xcgrp->leave();

  return;
}


void XtMain::doKeepAlive()
{
  xcgrp->join();
  return;
}


void XtMain::doUpdate()
{
  xcgrp->show();
  return;
}


void XtMain::setGroupName(QString val)
{
  conf->setGroupName(val);
  XtMainGroupNameValue->setText(conf->getGroupName());
  XtMainGroupNameValue->repaint();
  return;
}


void XtMain::setGroupServer(QString val)
{
  conf->setGroupServer(val);
  XtMainGroupServerValue->setText(conf->getGroupServer());
  XtMainGroupServerValue->repaint();
  return;
}


void XtMain::setGroupPort(Q_UINT16 val)
{
  conf->setGroupPort(val);
  XtMainGroupPortValue->setText(QString::number(conf->getGroupPort(), 10));
  XtMainGroupPortValue->repaint();
  return;
}


void XtMain::setVicCmd(QString val)
{
  conf->setVicCmd(val);
  return;
}


void XtMain::setVicPort(Q_UINT16 val)
{
  conf->setVicPort(val);
  return;
}


void XtMain::setVicTTL(Q_UINT16 val)
{
  conf->setVicTTL(val);
  return;
}


void XtMain::setRatCmd(QString val)
{
  conf->setRatCmd(val);
  return;
}


void XtMain::setRatPort(Q_UINT16 val)
{
  conf->setRatPort(val);
  return;
}


void XtMain::setRatTTL(Q_UINT16 val)
{
  conf->setRatTTL(val);
  return;
}


void XtMain::setKeepAliveTime(Q_UINT32 val)
{
  if(conf->getKeepAliveTime() != val){
    conf->setKeepAliveTime(val);
    keepAliveTimer->stop();
    keepAliveTimer->start((val*1000), FALSE);
  }
  return;
}


void XtMain::setUpdateTime(Q_UINT32 val)
{
  if(conf->getUpdateTime() != val){
    conf->setUpdateTime(val);
    updateTimer->stop();
    updateTimer->start((val*1000), FALSE);
  }
  return;
}


void XtMain::setIfName(QString val)
{
  conf->setIfName(val);
  return;
}


void XtMain::XtMainConfBtn_clicked()
{
  if(!JoinMember->childCount()){
    showMsgDlg("No Member Found");
    return;
  }

  QListViewItem *mbr = 0;
  XtMember member;
  mbr = JoinMember->selectedItem();
  if(mbr == 0){
    showMsgDlg("No Member Selected");
    return;
  }else{
    member.addr = mbr->text(0);
    if(mbr->text(1) == "yes")
     member.send = TRUE;
    else
     member.send = FALSE;
  }

  XtMbrConf *mbrConf = new XtMbrConf();

  /* Register SLOT (XtMbrConf -> XtMain) */
  connect(mbrConf, SIGNAL(sigSendMbrConf(XtMember)),
          this, SLOT(recvMbrConf(XtMember)));

  mbrConf->setMember(member);
  mbrConf->show();
  return;
}


void XtMain::recvMbrConf(XtMember member)
{
  bool found = FALSE;

  QListViewItemIterator iter(JoinMember->firstChild());
  for(; iter.current(); ++iter){
    if(iter.current()->text(0) == member.addr){
      found = TRUE;
      break;
    }
  }

  if(!found){
    showMsgDlg("Specified User NOT Exists");
    return;
  }

  QString flag;
  if(member.send)
    flag = "yes";
   else
    flag = "no";

  iter.current()->setText(1, flag);
  emit sigConfMember(member);

  return;
}


void XtMain::addMember(XtMember member)
{
  bool found = FALSE;
  QListViewItem *preItem = 0;

  QListViewItemIterator it(JoinMember->firstChild());
  for(; it.current(); ++it){
    if(it.current()->text(0) == member.addr){
      found = TRUE;
      break;
    }
    preItem = it.current();
  }

  if(found){
    showMsgDlg("Member Already Exists");
    return;
  }

  QListViewItem *entry;
  if(preItem == 0)
    entry = new QListViewItem(JoinMember);
  else
    entry = new QListViewItem(JoinMember, preItem);

  entry->setText(0, member.addr);

  QString flag;
  if(member.send)
    flag = "yes";
  else
    flag = "no";

  entry->setText(1, flag);

  return;
}


void XtMain::addToMember(XtMember member, XtMember after)
{
  bool found = FALSE;

  QListViewItemIterator it(JoinMember->firstChild());
  for(; it.current(); ++it){
    if(it.current()->text(0) == member.addr){
      found = TRUE;
      break;
    }
  }

  if(found){
    showMsgDlg("Member Already Exists");
    return;
  }

  QListViewItem *preItem = 0;
  QListViewItem *afterItem = 0;
  it = JoinMember->firstChild();
  for(; it.current(); ++it){
    if(it.current()->text(0) == after.addr){
      afterItem = it.current();
      break;
    }
    preItem = it.current();
  }

  QListViewItem *entry;
  if(afterItem == 0){
    if(preItem == 0)    /* add the beginning of list */
      entry = new QListViewItem(JoinMember);
    else                /* add the end of list */
      entry = new QListViewItem(JoinMember, preItem);
  }else                 /* add after "after" item */
      entry = new QListViewItem(JoinMember, afterItem);

  entry->setText(0, member.addr);

  QString flag;
  if(member.send)
    flag = "yes";
  else
    flag = "no";

  entry->setText(1, flag);

  return;
}


void XtMain::delMember(QString addr)
{
  bool found = FALSE;

  QListViewItemIterator it(JoinMember->firstChild());
  for(; it.current(); ++it){
    if(it.current()->text(0) == addr){
      found = TRUE;
      break;
    }
  }

  if(!found){
    showMsgDlg("Specified User NOT Exists");
    return;
  }

  JoinMember->takeItem(it.current());

  return;
}


void XtMain::updateMember(QValueList<XtMember> mbrList)
{
  JoinMember->clear();

  QListViewItem *preItem = 0;
  QValueList<XtMember>::iterator it;
  for(it = mbrList.begin(); it != mbrList.end(); ++it){
    QListViewItem *entry;
    if(preItem == 0) 
      entry = new QListViewItem(JoinMember);
    else
      entry = new QListViewItem(JoinMember, preItem);

    entry->setText(0, (*it).addr);

    QString flag;
    if((*it).send)
      flag = "yes";
    else
      flag = "no";

    entry->setText(1, flag);
    preItem = entry;
  }

  return;
}


bool XtMain::checkFile( QString file )
{
  QFile filePath(file);
  if(filePath.exists() == FALSE){
    QString reason("No such file: \n");
    reason += file;
    showMsgDlg((char *)reason.data());
    return FALSE;
  }

  return TRUE;
}


void XtMain::showMsgDlg(char *msg)
{
  QString reason(msg);
  XtMsgDlg *msgDlg = new XtMsgDlg(this);
  msgDlg->setMsg(reason);
  msgDlg->show();
  return;
}


