/* $Id: XtMbus.cpp,v 1.1 2006/02/02 15:37:57 jsato Exp $ */
/*
 * Copyright (C) 2006 WIDE Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * 2006/02/03 jsato: created
 *
 */


#include <stdlib.h>

#include "mbus.h"
#include "qxcgroup.h"
#include "XtMbus.h"


static xmb_t *xmb;
static void xmbInit(const char *);
static void xmbCmdHandler(char *, char *, char *, void *);
static void xmbErrHandler(int, int);
static void xmbSend(void);
static int  xmbRecv(unsigned int);
static void xmbQmsg(char *, char *, char *, int);
static void xmbHeartBeat(unsigned int);
static void xmbRetransmit(void);


/*
 * Definitions of "XtMbus" Class
 */

XtMbus::XtMbus(QString mbusAddr)
{
  xmbAddr   = mbusAddr;
  xmbMember = QString("");

  xmbInit((const char *)xmbAddr.data());
  xmbHeartBeat(1);
}


XtMbus::~XtMbus()
{
  free(xmb);
}


void XtMbus::keepalive()
{
  xmbHeartBeat(1);
//  xmbRecv(, 50);
  xmbRetransmit();
  return;
}


void XtMbus::announce(QString mbrList)
{
  xmbMember = mbrList;
  xmbQmsg("()", "xcgroup.member", (char *)xmbMember.data(), FALSE);
  xmbSend();
  return;
}



/*
 * Definitions of mbus function
 */

static void xmbInit(const char *xmbAddr)
{
  MSGDBG("xmbAddr=%s", xmbAddr);

  xmb = (xmb_t *)malloc(sizeof(xmb_t));
  memset(xmb, 0, sizeof(xmb_t));
  xmb->mb = mbus_init(&xmbCmdHandler,&xmbErrHandler,(char *)xmbAddr);
  return;
}


static void xmbCmdHandler(char *src, char *cmd, char *arg, void *dat)
{
  MSGDBG("src=%s dat=%p %s %s", src, dat, cmd, arg);

  return;
}


static void xmbErrHandler(int seq, int reason)
{
  MSGDBG("seq=%d reason=%d", seq, reason);

  return;
}


static void xmbSend(void)
{
  MSGDBG("called");

  mbus_send(xmb->mb);
  return;
}


static int xmbRecv(char *memList, unsigned int timeout)
{
  int t = 0;
  struct timeval tv;

  MSGDBG("memList=%s, timeout=%d", memList, timeout);

  t = (int)timeout;
  tv.tv_sec  = t / 1000;
  tv.tv_usec = (t % 1000) * 1000;

  return mbus_recv(xmb->mb, memList, &tv);
}


static void xmbQmsg(char *dst, char *cmd, char *arg, int reliable)
{
  MSGDBG("dst=%s cmd=%s arg=%s reliable=%d", dst, cmd, arg, reliable);

  mbus_qmsg(xmb->mb,(const char *)dst, (const char *)cmd,
            (const char *)arg, reliable);
  return;
}


static void xmbHeartBeat(unsigned int interval)
{
  MSGDBG("interval=%d", interval);

  mbus_heartbeat(xmb->mb, (int)interval);
  return;
}


static void xmbRetransmit(void)
{
  MSGDBG("called");

  mbus_retransmit(xmb->mb);
  return;
}
