/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
/* $Id: XtPref.ui.h,v 1.1 2006/02/02 15:37:57 jsato Exp $ */
/*
 * Copyright (C) 2006 WIDE Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * 2006/02/03 jsato: created
 *
 */



void XtPref::setConf(XtConf *conf)
{
  xtConf = conf;

  XtGroupNameValue->setText(xtConf->getGroupName());
  XtGroupServerValue->setText(xtConf->getGroupServer());
  XtGroupPortValue->setText(QString::number(xtConf->getGroupPort(), 10));
  XtVicCmdValue->setText(xtConf->getVicCmd());
  XtVicPortValue->setText(QString::number(xtConf->getVicPort(), 10));
  XtVicTTLValue->setText(QString::number(xtConf->getVicTTL(), 10));
  XtRatCmdValue->setText(xtConf->getRatCmd());
  XtRatPortValue->setText(QString::number(xtConf->getRatPort(), 10));
  XtRatTTLValue->setText(QString::number(xtConf->getRatTTL(), 10));
  XtKeepAliveValue->setText(QString::number(xtConf->getKeepAliveTime(), 10));
  XtUpdateTimeValue->setText(QString::number(xtConf->getUpdateTime(), 10));

  /* Prepare items of active interface name */
  QValueList<QString> allIf = xtConf->getIfNameAll();
  if(allIf.isEmpty() == TRUE){
    showMsgDlg("No Valid Interface Found");
  }else{
    QValueList<QString>::iterator it;
    for(it = allIf.begin(); it != allIf.end(); it++)
      XtIfNameValue->insertItem((*it), -1);

    QString ifname = xtConf->getIfName();
    if(ifname.isEmpty() == TRUE)
      ifname = XtIfNameValue->text(0);
    else
      XtIfNameValue->setCurrentText(ifname);

    XtIfAddrValue->setText(xtConf->getIfAddr(ifname));
  }

  return;
}


void XtPref::XtPrefOkBtn_clicked()
{
  Q_UINT16 grp_port = XtGroupPortValue->text().toUShort();
  if(grp_port == 0 || grp_port > 65535){
    showMsgDlg("Invalid xcgroup Port Range");
    return;
  }

  Q_UINT16 vic_port = XtVicPortValue->text().toUShort();
  if(vic_port == 0 || vic_port > 65535){
    showMsgDlg("Invalid vic Port Range");
    return;
  }

  Q_UINT16 vic_ttl = XtVicTTLValue->text().toUShort();
  if(vic_ttl == 0 || vic_ttl > 256){
    showMsgDlg("Invalid vic TTL Range");
    return;
  }

  Q_UINT16 rat_port = XtRatPortValue->text().toUShort();
  if(vic_port == 0 || vic_port > 65535){
    showMsgDlg("Invalid rat Port Range");
    return;
  }

  Q_UINT16 rat_ttl = XtRatTTLValue->text().toUShort();
  if(rat_ttl == 0 || rat_ttl > 128){
    showMsgDlg("Invalid rat TTL Range");
    return;
  }

  Q_UINT32 keep_alive = XtKeepAliveValue->text().toUInt();
  if(keep_alive <= 9){
    showMsgDlg("Keep-Alive Time must be greater than 9 sec");
    return;
  }else if(keep_alive > (unsigned short)~0){
    showMsgDlg("Keep-Alive Time must be lesser than 65535 sec");
    return;
  }

  Q_UINT32 update_time = XtUpdateTimeValue->text().toUInt();
  if(update_time <= 9){
    showMsgDlg("Update Time must be greater than 9 sec");
    return;
  }else if(update_time > (unsigned short)~0){
    showMsgDlg("Update Time must be lesser than 65535 sec");
    return;
  }

  emit sigSetGroupName(XtGroupNameValue->text());
  emit sigSetGroupServer(XtGroupServerValue->text());
  emit sigSetGroupPort(grp_port);
  emit sigSetVicCmd(XtVicCmdValue->text());
  emit sigSetVicPort(vic_port);
  emit sigSetVicTTL(vic_ttl);
  emit sigSetRatCmd(XtRatCmdValue->text());
  emit sigSetRatPort(rat_port);
  emit sigSetRatTTL(rat_ttl);
  emit sigSetKeepAliveTime(keep_alive);
  emit sigSetUpdateTime(update_time);

  emit sigSetIfName(XtIfNameValue->currentText());

  this->close();
  return;
}


void XtPref::XtIfNameValue_activated( int index )
{
  QString ifname(XtIfNameValue->text(index));
  XtIfAddrValue->setText(xtConf->getIfAddr(ifname));
  return;
}


void XtPref::showMsgDlg(char *msg)
{
  QString reason(msg);
  XtMsgDlg *msgDlg = new XtMsgDlg(this);
  msgDlg->setMsg(reason);
  msgDlg->show();
  return;
}
