/* $Id: XtXcgrp.h,v 1.1 2006/02/02 15:37:57 jsato Exp $ */
/*
 * Copyright (C) 2006 WIDE Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * 2006/02/03 jsato: created
 *
 */


#ifndef XT_XCGRP_H
#define XT_XCGRP_H

#include <netinet/in.h>

#include <qobject.h>
#include <qstring.h>

#include "qxcgroup.h"
#include "XtConf.h"
#include "XtHttp.h"
#include "XtMbus.h"


#define  MBR_SCOPE_OWN          1   /* The member is own */
#define  MBR_SCOPE_LINK         2   /* The member is in same link */
#define  MBR_SCOPE_SITE         3   /* The member is in same network */
#define  MBR_SCOPE_OTHER        4   /* The member is in other network */



class XtMember : public QString
{
public:
  QString addr;
  bool    send;
  bool    alive;
  Q_UINT8 scope;
};


class XtXcgrp : public QObject
{
  Q_OBJECT

public:
  XtXcgrp(XtConf *);
  ~XtXcgrp();

  void join();
  void leave();
  void show();

public slots:
  void httpRecvData(const char *data, Q_ULONG data_len);
  void httpRecvEnd();
  void httpRecvErr(Q_UINT8 ret);
  void confMember(XtMember member);
  void confUpdate();

signals:
  void sigAddMember(XtMember member);
  void sigAddToMember(XtMember member, XtMember after);
  void sigDelMember(QString addr);
  void sigUpdateMember(QValueList<XtMember>);
  void sigUpdateHttp(QString, Q_UINT16);
  void sigSendErr(int);

private:
  XtConf    *xtConf;
  QString   grpName;
  QString   httpServer;
  Q_UINT16  httpPort;
  QString   mbusAddr;
  QString   ownAddr;
  Q_UINT16  ownPrefix;
  sockaddr_in6 ownSock;
  QString   baseUrl;
  QValueList<XtMember> mbrList;
  Q_UINT8   mbrNum;
  XtHttp    *grpOps;
  XtHttp    *grpGet;
  XtMbus    *mbusHdl;

  void      add_member(QString member);
  void      addto_member(QString member, QString after);
  void      del_member(QString member);
  void      un_member(QString member, bool flag);
  bool      is_member(QString member);
  XtMember  *get_member(QString member);
  QString   get_time();
  Q_UINT8   get_scope(QString addr);
#ifdef QXCGROUP_MEMBER_SORT
  void      sort_member();
#endif /* QXCGROUP_MEMBER_SORT */
};

#endif /* XT_XCGRP_H */
