/* $Id: qxcgroup.h,v 1.1 2006/02/02 15:37:57 jsato Exp $ */
/*
 * Copyright (C) 2006 WIDE Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * 2006/02/03 jsato: created
 *
 */


#ifndef QXCGROUP_H
#define QXCGROUP_H

#include "mbus.h"


#define QXCGROUP_VERSION    "0.1"


/*
 * Definition of Macros 
 */

/* various default values */
#define DEF_XCGRP_PORT      80
#define DEF_XCGRP_CMD       "/usr/bin/xcgroup"
#define DEF_VIC_PORT        11000
#define DEF_VIC_CMD         "/usr/bin/vic"
#define DEF_RAT_PORT        11111
#define DEF_RAT_CMD         "/usr/bin/rat"
#define DEF_TTL             64
#define DEF_KALIVE_TIME     10
#define DEF_UPDATE_TIME     10
#define DEF_OS_TYPE         "linux" /* "win32"or"bsd" are allowed as others */ 


/* Definition of using in xcgroup */
#define XCGRP_URL           "/cgi-bin/xcgroupsrv.cgi?group=%s&cmd="
#define XCGRP_MBUS          "(media:control module:control app:xcgroup id:%u)"


/* xcgroup error code */
#define XCGRP_OK            0
#define XCGRP_ERR           1
#define XCGRP_E_NO_MBR      2
#define XCGRP_E_MBR_EXIST   3
#define XCGRP_E_SOCKET      4
#define XCGRP_E_CONNECT     5
#define XCGRP_E_SEND        6
#define XCGRP_E_RECV        7
#define XCGRP_E_BAD_ADDR    8
#define XCGRP_E_NO_SOCK     9
#define XCGRP_E_HTTP        10
#define XCGRP_E_HTTP_400    40
#define XCGRP_E_HTTP_401    41
#define XCGRP_E_HTTP_402    42
#define XCGRP_E_HTTP_403    43
#define XCGRP_E_HTTP_404    44
#define XCGRP_E_HTTP_405    45
#define XCGRP_E_HTTP_406    46
#define XCGRP_E_HTTP_407    47
#define XCGRP_E_HTTP_408    48
#define XCGRP_E_HTTP_500    50
#define XCGRP_E_HTTP_501    51
#define XCGRP_E_HTTP_502    52
#define XCGRP_E_HTTP_503    53
#define XCGRP_E_HTTP_504    54
#define XCGRP_E_HTTP_505    55
#define XCGRP_E_HTTP_BUSY   60


/*
 * Options
 */


/* for debugging */
#ifdef QXCG_DEBUG
#define QXCGROUP_DEBUG          1
#endif 


/* JSATO - the following is test option */
#ifdef QXCG_MEMBER_SORT
#define QXCGROUP_MEMBER_SORT    1
#endif


typedef struct {
    struct mbus *mb;
    void   *data;
} xmb_t;



/*
 * Logging macros
 */

extern int verb;
#define MSGVRB(fmt,arg...) \
do{ \
  if(verb){ \
    fprintf(stderr, fmt, ## arg); \
    fprintf(stderr, "\n"); \
  } \
}while(0)

#define MSGERR(fmt,arg...) \
do{ \
  fprintf(stderr, "***ERROR*** "); \
  fprintf(stderr, fmt, ## arg); \
  fprintf(stderr, "\n"); \
}while(0)

#ifdef QXCGROUP_DEBUG
#define MSGDBG(fmt, arg...) \
do{ \
  fprintf(stderr, "[qxcgroup:%s] ", __FUNCTION__); \
  fprintf(stderr, fmt, ## arg); \
  fprintf(stderr, "\n"); \
}while(0)
#else
#define MSGDBG(fmt, arg...)         do{}while(0)
#endif /* QXCGROUP_DEBUG */

#endif /* QXCGROUP_H */
